#include <htc.h>
#include "wait_time.h"
#include "user_define.h"
#include "cpld.h"

#define	LCD_DB4				RB7
#define	LCD_DB5				RB6
#define	LCD_DB6				RB1
#define	LCD_DB7				RB0
#define	LCD_RS				RB4
#define	LCD_E				RB5	

#define	LCD_CMD_WAIT		200  /*400*/
#define	LCD_BIT_WAIT		5   /*10*/

extern	unsigned char	freq_byte1;
extern	unsigned char	freq_byte2;
extern	unsigned char	freq_byte3;
extern	unsigned char	freq_byte4;
extern	unsigned char	mode;
extern	unsigned char	f_add;

/*---------------LCD MODULE----------------*/
void	lcd_db(unsigned char data)
{
	LCD_DB4=data&0x01;
	NOP();
	LCD_DB5=(data&0x02)>>1;
	NOP();
	LCD_DB6=(data&0x04)>>2;
	NOP();
	LCD_DB7=(data&0x08)>>3;
	wait_time(1);
}
void	lcd_wait(void)
{
	wait_time(2500);
}
void	lcd_c_set(void)
{
	LCD_RS=0;
	NOP();
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
	NOP();
}
void	lcd_d_set(void)
{
	LCD_RS=1;
	NOP();
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
	NOP();
}
void	lcd_cmd(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_c_set();
	lcd_db(data&0x0f);
	lcd_c_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_data(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_d_set();
	lcd_db(data&0x0f);
	lcd_d_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_init(void)
{
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(2);
	lcd_c_set();
	lcd_wait();
	
	lcd_cmd(0x28);
	lcd_cmd(0x08);
	lcd_cmd(0x0c);
	lcd_cmd(0x06);
}
void	lcd_clear(void)
{
	lcd_cmd(0x01);
}
void	lcd_xy(unsigned char x,unsigned char y)
{
unsigned char data;

	data=0x80;
	switch (y){
		case 0:data=0x80;break;
		case 1:data=0xc0;break;
		case 2:data=0x90;break;
		case 3:data=0xd0;break;
	}
	data=data+x;
	lcd_cmd(data);
}
void	lcd_print(const char *data)
{
	while (*data!=0){
		lcd_data(*data++);
	}
}
void	lcd_su(unsigned char data)
{
	switch (data){
		case 0:lcd_print("0");break;
		case 1:lcd_print("1");break;
		case 2:lcd_print("2");break;
		case 3:lcd_print("3");break;
		case 4:lcd_print("4");break;
		case 5:lcd_print("5");break;
		case 6:lcd_print("6");break;
		case 7:lcd_print("7");break;
		case 8:lcd_print("8");break;
		case 9:lcd_print("9");break;
		case 10:lcd_print("A");break;
		case 11:lcd_print("B");break;
		case 12:lcd_print("C");break;
		case 13:lcd_print("D");break;
		case 14:lcd_print("E");break;
		case 15:lcd_print("F");break;
	}
}
void	lcd_pr_byte(unsigned char data)
{
unsigned char data2;

	data2=data%16;data=data/16;
	lcd_su(data);
	lcd_su(data2);
}
void	lcd_1keta10(unsigned char count)
{
unsigned char n0;

	n0=(unsigned char)(count%10);

	lcd_su(n0);
}

void	lcd_2keta10(unsigned char count)
{
unsigned char n0,n1;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

	lcd_su(n1);
	lcd_su(n0);
}

void	lcd_mhz(unsigned int hz)
{
unsigned char n0,n1,n2,n3,n4;
unsigned int count;

	count=hz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);count=(count-n3)/10;
	n4=(unsigned char)(count%10);
	
	if(n4==0){lcd_print(" ");}else{lcd_su(n4);}
	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);	
}

void	lcd_freq(void)
{
unsigned char	n0,n1,n2,n3,n4,n5,n6;

	n0=freq_byte1 & 0xf0;n0=n0 >> 4;
	n1=freq_byte1 & 0x0f;
	n2=freq_byte2 & 0xf0;n2=n2 >> 4;
	n3=freq_byte2 & 0x0f;
	n4=freq_byte3 & 0xf0;n4=n4 >> 4;
	n5=freq_byte3 & 0x0f;
	n6=freq_byte4 & 0x0f;

	if(n1==0){lcd_print(" ");}else{lcd_su(n1);}
	lcd_su(n0);
	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n5);
	lcd_su(n4);
	lcd_print(".");
	lcd_su(n6);

}

void	lcd_mode(void)
{
	lcd_xy(0,1);
	switch(mode){
		case 0:	lcd_print("AIR  ");break;
		case 1:	lcd_print("NORM ");break;
	}
	lcd_xy(3,1);
	if(mode==0){
		switch(f_add){
			case 0:	lcd_print("+");break;
			case 1:	lcd_print(" ");break;
		}
	}
}